import java.io.*;
import java.util.Vector;

/**
 * Contains convenience methods for file operations
 */
public class FileIO
{

  /**
   *
   */
  private FileIO()
  {
  }

  /**
   * Creates the only instance of this class
   */
  public static FileIO instance()
  {
    if (_instance == null)
    {
      _instance = new FileIO();
    }
    return _instance;
  }


  /**
   * Reads text file to <code>InputStream</code>
   * @param file the source file
   * @return file contents
   */
   public InputStream readText(File file)
   {
    FileInputStream result=null;
    try
    {
      if(file.exists() && file.canRead())
      {
        result = new FileInputStream(file);
      }
      else
      {
        System.out.println("Read Text: Cannot read from file: "+file.getAbsolutePath());
      }
    }
    catch (IOException e)
    {
      System.out.println("Read Text: I/O Error ! "+e.getMessage());
    }
    return result;
   }


  /**
   * Writes <code>Object</code> Array to text file.
   * If the file exists, it will be replaced
   * <br>Writes 1 line for each <code>Object</code>
   * @param file the destination file
   * @param text the Array containing the lines to be written
   * must contain <code>String objects</code>
   */
  public void writeText(File file, Object[] text)
  {
    try
    {
      if(file.exists() && file.canWrite())
      {
        BufferedWriter buf = new BufferedWriter(new FileWriter(file));
        PrintWriter out = new PrintWriter(buf);

        for(int i=0;i<text.length-1;i++)
        {
          out.write((String) text[i]);
          buf.newLine();
        }
        out.write((String) text[text.length-1]);
        out.close();
      }
      else if(!file.exists())
      {
        BufferedWriter buf = new BufferedWriter(new FileWriter(file));
        PrintWriter out = new PrintWriter(buf);

        for(int i=0;i<text.length;i++)
        {
          out.write((String) text[i]);
          buf.newLine();
        }
        out.close();
      }
      else
      {
        System.out.println("Write Text: Unable to write Text to "+ file.getAbsolutePath());
      }
    }
    catch (IOException e)
    {
      System.out.println("Write Text: I/O Error ! "+e.getMessage());
    }
    catch(ClassCastException e)
    {
      System.out.println("Write Text: Wrong Class Type ! "+e.getMessage());
    }
  }

  /**
   * Writes an <code>Object</code> to a file.
   * <br>If the file exists, it will be replaced
   * @param saveMe the <code>Object</code> to be written
   * @param file the file to be written into
   */
  public void writeObject(Object saveMe,File file)
  {
    try
    {
      if(file.exists() && file.canWrite())
      {
        FileOutputStream fileOut = new FileOutputStream(file);
        ObjectOutputStream out = new ObjectOutputStream(new BufferedOutputStream(fileOut));

        out.writeObject(saveMe);
        out.flush();
        out.close();
      }
      else if(!file.exists())
      {
        FileOutputStream fileOut = new FileOutputStream(file);
        ObjectOutputStream out = new ObjectOutputStream(new BufferedOutputStream(fileOut));

        out.writeObject(saveMe);
        out.flush();
        out.close();
      }
      else
      {
        System.out.println("Write Object: Unable to write "+ saveMe.getClass()+" to "+ file.getAbsolutePath());
      }
    }
    catch (IOException e)
    {
      System.out.println("Write Object: I/O Error ! "+e.getMessage());
    }
  }

  /**
   * Reads an <code>Object</code> from a file
   * @param file the source file
   * @return <code>Object</code> serialized in the file
   */
  public Object readObject(File file)
  {
    Object result=null;
    try
    {
      if(file.exists() && file.canRead())
      {
        FileInputStream fileIn = new FileInputStream(file);
        ObjectInputStream in = new ObjectInputStream(new BufferedInputStream(fileIn));

        result = (Object) in.readObject();
      }
      else
      {
        System.out.println("Read Object: Unable to read file: "+ file.getAbsolutePath());
      }
    }
    catch(EOFException e)
    {
      System.out.println("Read Object: "+e.getMessage()+" !");
    }
    catch (IOException e)
    {
      System.out.println("Read Object: I/O Error ! "+e.getMessage());
    }
    catch(ClassNotFoundException e)
    {
      System.out.println("Read Object: Class not Found ! "+e.getMessage());
    }
    return result;
  }
  private static FileIO _instance;

}