// hier Package-Definitioneinfuegen, z.B. package syntax;


import java.util.TreeMap;

/**
 * Interface for communicating with the CDG-Parser
 */
public interface SyntaxDiagnose
{

    /**
     * Analyses the german natural language user input
     * for grammatical correctness
     * @param sentence the sentence input by user
     */
    public void analyze(String sentence);

    /**
     * The natural language sentence input by the user was correct
     * @return the sentence was correct
     */
    public boolean isCorrect();

    /**
     * Gives an explanation for incorrect sentences
     * <br>!!!return type may change!!!
     * @return explanation of the error containing a list of violations
     */
    public Explanation explanation();

    /**
     * Evaluation of the quality of an input sentence
     * @return quality of input sentence - 0.0 if sentence was correct
     */
    public double quality();

    /**
     * Returns syntax tree generated by the CDG-Parser
     * <br>!!!return type may change!!!
     * @return the syntax tree generated by the CDG-Parser
     */
    public TreeMap syntaxTree();

}
