// hier Package-Definitioneinfuegen, z.B. package syntax;


import java.util.TreeMap;

/**
 * Dummy implementation of interface <code>SyntaxDiagnose</code>
 * for communicating with the CDG-Parser
 */
public class SyntaxDiagnoseImpl implements SyntaxDiagnose
{

    /**
     *
     */
    public SyntaxDiagnoseImpl()
    {
	explanation = new Explanation();
    }

    /**
     * Analyzes the german natural language user input
     * for grammatical correctness
     * @param sentence the sentence input by user
     */
    public void analyze(String sentence)
    {
	explanation = new Explanation();
	quality = 0.0;

	if (sentence.toLowerCase().equals(SATZ3)
	    ||
	    sentence.toLowerCase().equals(SATZ4))
	    {
		explanation.addViolation("ERR_AGREEMENT: das=neutr, hocker=masc");
		quality=0.02;
	    }
    }

    /**
     * The natural language sentence input by the user was correct
     * @return the centence was correct
     */
    public boolean isCorrect()
    {
	return !explanation.violationFound();
    }

    /**
     * Gives an explanation for incorrect sentences
     * @return explanation of the error
     */
    public Explanation explanation()
    {
	return explanation;
    }

    /**
     * Returns syntax tree generated by the CDG-Parser
     * @return the syntax tree generated by the CDG-Parser
     */
    public TreeMap syntaxTree()
    {
	return new TreeMap();
    }

    /**
     * evaluation of the quality of an input sentence
     * @return quality of input sentence - 0.0 if sentence was correct
     */
    public double quality()
    {
	return quality;
    }

    // input
    private String sentence;

    // output

    private TreeMap syntaxTree;

    private Explanation explanation;

    private double quality;

    private final String SATZ1="der hocker steht unter dem tisch";
    private final String SATZ2="der hocker steht auf dem tisch";
    private final String SATZ3="das hocker steht unter dem tisch";
    private final String SATZ4="das hocker steht auf dem tisch";



}
