// hier Package-Definitioneinfuegen, z.B. package syntax;


import java.net.*;
import java.io.*;
import java.util.Properties;

/**
 * Socket class for <code>SyntaxDiagnose</code>
 * <p>Gets socket # from file <code>serverConfig.txt</code> which must be placed in the classpath
 * <br>Accepts only 1 client at a time.
 * <br>Exit this program by sending "shutdown"
 */
public class SyntaxSocket
{
    /**
     * Reads socket No. from <code>serverConfig.txt</code> & creates a <code>ServerSocket</code>
     * <br>If there is no <code>serverConfig.txt</code> starts with default socket # 4711
     */
    public SyntaxSocket() throws IOException {

      // get socket # from serverConfig.txt
      Properties properties = new Properties();
      InputStream in = FileIO.instance().readText(new File("serverConfig.txt"));

      try {
        properties.load(in);
        String socketNo = properties.getProperty("socket");
        _socket = new ServerSocket(new Integer(socketNo).intValue());
        System.out.println("Socket # "+socketNo);
      }
      catch(NullPointerException e)
      {
        System.out.println("No serverConfig.txt! Server set to default socket # 4711");
        _socket = new ServerSocket(4711);
      }
    }

    /**
     * Creates a new <code>SyntaxSocket</code> and waits for client input.
     * <br>Accepts only 1 client at a time.
     * <br>Exit this program by sending "shutdown"
     */
    public static void main(String[] args) throws IOException {

	_syntax = new SyntaxSocket();
	System.out.println("Server started!");
	PrintWriter out;
	BufferedReader in;
	String inputLine="";
	String outputLine="";

	while(true) {
            try {
		// wait for new client
		_clientSocket=_socket.accept();
		System.out.println("incoming!");

		// initialize I/O streams
		out = new PrintWriter(_clientSocket.getOutputStream(), true);
		in = new BufferedReader(new InputStreamReader(_clientSocket.getInputStream()));

		// initiate conversation with client

		out.println("okeydokey!");

		// react on client input until client exits
		while ((inputLine = in.readLine()) != null) {
                    System.out.println("received \""+inputLine+"\" from client");
 		    // if client sends "shutdown", exit program
		     if (inputLine.equals("shutdown")) {
			 System.exit(0);
		     }
                     else if(inputLine.equals("exit"))
                     {
                      System.out.println("Client will exit");
                     }
                      else
                      {
                        System.out.println("... analyzing ...");
		        String reply = analyze(inputLine);
                        System.out.println("... ready!");
		        out.println(reply);
                      }
		}
            }
            catch(SocketException e)
            {
              System.out.println("Client gone to nirvana");
            }
        }
    }

    /**
     * Calls the <code>SyntaxDiagnose</code> interface and
     * <br>Creates a <code>String</code> representation of the diagnosis
     * @param sentence user input to be analyzed
     * @return String representation of the diagnosis result
     */
    private static synchronized String analyze(String sentence)
    {
	SyntaxDiagnose diag = new SyntaxDiagnoseImpl();
	Explanation explanation = new Explanation();
	Object[] violations;

	String reply ="[User Input: "+sentence +"]";
	diag.analyze(sentence);
	reply = reply + "[korrekt?  "+diag.isCorrect() +"]";

	if (!diag.isCorrect())
	    {
		explanation = diag.explanation();
		violations = explanation.getViolations();
		reply = reply + "[Number of Violations: "+violations.length+"]";

		for(int i=0;i<violations.length;i++)
		    {
			reply = reply + "[Violation: "+(String) violations[i]+"]";
		    }
	    }

	reply = reply + "[quality: "+diag.quality() +"]";
	reply = reply + "["+diag.syntaxTree().toString()+"]";
	return reply;
    }



    private static ServerSocket _socket;
    private static SyntaxSocket _syntax;
    private static Socket _clientSocket = null;
}
