// hier Package-Definitioneinfuegen, z.B. package syntax;


import java.util.TreeMap;
import java.net.*;
import java.io.*;
import java.util.Properties;

/**
 * Test client for <code>SyntaxSocket</code>
 * <p>Gets host name & socket # from file <code>clientConfig.txt</code> which
 * must be placed in the <code>classpath</code>
 * <br>Exit client with ^C or "exit"
 * <br>Shutdown server & exit client with "shutdown"
 */
public class TestClient {

    /**
     * Gets host name and socket # from <code>clientConfig.txt</code>.
     * <br>Gets the server at host name & socket #
     * <br>If there is no <code>clientConfig.txt</code> gets the server at
     * default "localhost", socket #4711
     * <br>Exit client with ^C or "exit"
     * <br>Shutdown server & exit client with "shutdown"
     */
    public static void main(String[] args) throws java.net.UnknownHostException, java.io.IOException {

      // make new client socket
      try {
      Socket socket;
        try {
          // get host name & socket # from clientConfig.txt
          Properties properties = new Properties();
          InputStream in = FileIO.instance().readText(new File("clientConfig.txt"));
          properties.load(in);
          String socketNo = properties.getProperty("socket");
          String host = properties.getProperty("host");

          socket = new Socket(host,new Integer(socketNo).intValue());
          System.out.println("Host "+host+"\nSocket # "+socketNo);
      }
      catch(NullPointerException e)
      {
        System.out.println("No clientConfig.txt! Server set to default socket # 4711 on localhost");
        socket = new Socket("localhost",4711);
      }

	// initialize I/O Streams
	PrintWriter out = new PrintWriter(socket.getOutputStream(), true);
        BufferedReader in = new BufferedReader(new InputStreamReader(socket.getInputStream()));

	BufferedReader stdIn = new BufferedReader(new InputStreamReader(System.in));

	// read user keyboard input until user terminates program
	String fromServer="";
        String fromUser="";
	while (fromServer != null) {
          try {
            fromServer = in.readLine();
	    System.out.println("Server: " + fromServer);

	    fromUser = stdIn.readLine();
	    if (fromUser != null) {
		System.out.println("Client: " + fromUser);
		out.println(fromUser);
	    }
	    if(fromUser.equals("shutdown") || fromUser.equals("exit")) {
		out.close();
		in.close();
		stdIn.close();
		socket.close();
		System.exit(0);
	    }
          }
          catch(SocketException e)
          {
            System.out.println("Server gone to nirvana");
            fromServer=null;
          }
	}

	// please be tidy
	out.close();
	in.close();
	stdIn.close();
	socket.close();
      }

      catch(ConnectException e)
      {
        System.out.println("No connection to server");
      }

    }
}
